/*
 * Afrimesh: easy management for B.A.T.M.A.N. wireless mesh networks
 * Copyright (C) 2008-2009 Meraka Institute of the CSIR
 * All rights reserved.
 *  
 * This software is licensed as free software under the terms of the
 * New BSD License. See /LICENSE for more information.
 */

var populate_dom = null;
var populate_select_interface = null;
var populate_mesh_controls = null;
var update_sip_server = null;
var update_iax_server = null;
var update_radius_server = null;
var update_vis_server = null;
var update_map_server = null;
var LocationMap = null;
(function() {

  /** populate DOM with the values in afrimesh.settings ------------------- */
  populate_dom = function() {
    var elements = $("[id*=afrimesh|settings]");
    elements.each(function(element) {
        element = elements[element];
        if (element instanceof HTMLSelectElement) { // TODO - handle select elements
        } else if (element instanceof HTMLInputElement) {
          if (element.type == "checkbox") {
            var checked = Q(afrimesh, element.id);
            if (checked == "on" || checked == "1" || checked == "true") {
              checked = true;
            } else {
              checked = false;
            }
            $(element).attr("checked", checked); 
          } else {
            var value = Q(afrimesh, element.id);
            if (value != undefined) {
              $(element).val(String(value)); 
            } else {
              $(element).val("");
            }
          }
        } else if (element instanceof HTMLElement) {
            var value = Q(afrimesh, element.id);
            if (value != undefined) {
              $(element).html(String(value)); 
            } else {
              $(element).html("");
            }
        } else {
          console.warn("UNKNOWN ELEMENT TYPE: " + element);
        }
      });
  };


  /** query internet gateway's interfaces via snmp ---------------------- */
  populate_select_interface = function() {
    $("input.[id*=afrimesh|settings|internet_gateway|address]").css("background", "#FFAAAA");
    var interfaces = afrimesh.villagebus.snmp.walk(afrimesh.settings.internet_gateway.address, 
                                                   afrimesh.settings.internet_gateway.snmp.community, 
                                                   ".1.3.6.1.2.1.2.2.1.2");  // IF-MIB::ifDescr
    console.log("GETTING INTERFACES: " + interfaces);
    if (!interfaces || interfaces.error) {
      $("p.[id*=internet_gateway|error]").html("Internet gateway unreachable.");
      console.debug("SNMP ERROR: " + (interfaces ? interfaces.error : "unknown error"));
      console.debug("dashboard server: " + afrimesh.settings.hosts.dashboard_server);
      console.debug("settings  server: " + afrimesh.settings.address);
      $("select.[id*=afrimesh|settings|internet_gateway|snmp|interface]").html("");
      return;
    }
    $("p.[id*=internet_gateway|error]").html("");
    $("input.[id*=afrimesh|settings|internet_gateway|address]").css("background", "#AAFFAA");
    var interface_current = afrimesh.settings.internet_gateway.snmp.interface;
    var interface_count = 1;
    var options = "";
    for (var interface in interfaces) {
      options += "<option value='" + interface_count + "' ";
      options += (interface_count == interface_current ? "selected" : "") + ">";
      options += interfaces[interface];
      options += "</option>";
      interface_count++;
    }
    $("select.[id*=afrimesh|settings|internet_gateway|snmp|interface]").html(options);
  }

  update_mesh_controls = function() {
    var routing_class = afrimesh.settings.network.mesh.routing_class;
    var gateway_class = afrimesh.settings.network.mesh.gateway_class;
    if (gateway_class != undefined && 
        gateway_class != "0" && 
        gateway_class != "") {
      $("input[name='is_mesh_gateway']").attr("checked", true);
    } else {
      $("input[name='is_mesh_gateway']").attr("checked", false);
      $("select.[id*=afrimesh|settings|network|mesh|routing_class]").val(routing_class);
    }
    $("input[name='is_mesh_gateway']").change();
  };
  
  update_sip_server = function() {
    $("fieldset#asterisk :text, :password").css("background", "#FFAAAA");
    var enabled = $("input.[id*=afrimesh|settings|voip|sip|enable]").attr("checked");
    if (!enabled) {
      $(".sip_trunk").hide();
      return;
    } 
    $(".sip_trunk").show();
    var server = $("input.[id*=afrimesh|settings|voip|sip|address]").val();
    afrimesh.villagebus.voip.sip.peers.async(function(peers) {
        peers.map(function(peer) {
            console.debug(show(peer));
            if (peer.host == server && peer.status == "OK") {
              $("fieldset#asterisk :text, :password").css("background", "#AAFFAA");
            }
          });
      }, afrimesh.settings.address);
  };

  update_iax_server = function() {
    //console.debug("checked: " + $("input.[id*=afrimesh|settings|voip|iax|enable]").is(":checked"));
    var enabled = $("input.[id*=afrimesh|settings|voip|iax|enable]").attr("checked");
    if (!enabled) {
      $(".iax_trunk").hide();
      return;
    } 
    $(".iax_trunk").show();
    var server = $("input.[id*=afrimesh|settings|voip|iax|address]").val();
    console.debug("asterisk server: " + server);
    // TODO - try to ping asterisk server
  };
  
  update_vis_server = function() {
    $("input.[id*=afrimesh|settings|network|mesh|vis_server]").css("background", "#FFAAAA");
    try {
      //var routes = afrimesh.villagebus.mesh_topology();
      function update (routes) {
        if (routes != undefined && isArray(routes)) {
          $("input.[id*=afrimesh|settings|network|mesh|vis_server]").css("background", "#AAFFAA");
          $("p.[id*=vis_server|error]").html("");
        } else {
          console.debug("utility.settings.js->update_vis_server: Visualization server unreachable.");
          $("p.[id*=vis_server|error]").html("Visualization server unreachable.");
        }
      };
      afrimesh.villagebus.mesh_topology.vis.async(update);
    } catch (error) {
      $("p.[id*=vis_server|error]").html("Visualization server unreachable. " + error + ".");
      console.debug("Vis server is unreachable. " + error);
    }
  };

  update_map_server = function() { 
    $("input.[id*=afrimesh|settings|map|server]").css("background", "#FFAAAA")
    $("#location").addClass("message");
    $("#location").html("<p><br/><p>Contacting Map Server...</p>");
    // if it's a valid map server it will have the OSM/OpenLayers scripts hosted
    try {
      //var request = $.get(afrimesh.villagebus.ajax_proxy() + "http://" + afrimesh.settings.map.server + "/openlayers/OpenStreetMap.js",
      //                    "", check_complete, "text");
      // TODO - cleanup. Disabled ^^^^ so we don't end up loading the OSM libs twice
      check_complete();
    } catch (error) {
      $("p.[id*=afrimesh|settings|map|server|error]").html("Map server unreachable: " + error + ".");
      console.debug("Map server unreachable: " + error);
      return;
    }

    function check_complete(data, textStatus) {
      if (data == "") {
        var message = "<p><br/><p>Could not contact Map Server.</p>";
        message += "<p><br/><p><b>Please check:</b> ";
        message += "The Map Server address and your network connection.";
        message += "</p>";
        $("#location").html(message);
        return;
      }

      // we have a map server
      $("#location").html("<p><br/><p>Map Server Contacted.</p> <p><br/><p>Loading Network Map...</p>");

      // Now Load OpenStreetMap & OpenLayers libraries
      load.async(afrimesh.settings.map.openlayers_url, function(data, textStatus) {
          if (typeof OpenLayers == "undefined") {
            return load_error("The OpenLayers mapping service could not be found.");
          }
          load.async(afrimesh.settings.map.osm_url, load_complete); // TODO - see above where we disabled double OSM load
        });
    };

    function load_complete(data, textStatus) {
      load("javascript/afrimesh.maps.js"); 
      $("p.[id*=map|server|error]").html("");
      $("#location").replaceWith("<div id='location' />");
      var location_map = new LocationMap("location", 
                                         parseFloat(afrimesh.settings.location.longitude),
                                         parseFloat(afrimesh.settings.location.latitude),
                                         parseFloat(afrimesh.settings.map.extent),
                                         parseInt(afrimesh.settings.map.zoom));
      var router = { address : afrimesh.settings.address };
      location_map.router(router);
      $("input.[id*=afrimesh|settings|map|server]").css("background", "#AAFFAA");
    };

    function load_error(message) {
      var html = "<p><br/><p>Could Not Contact Map Server.</p>";
      html += "<p><br/><p>" + message + "</p>";
      $("#location").html(html);
    };

  };
  
  update_radius_server = function() {
    var radtype_current = afrimesh.settings.radius.radtype;
    if (radtype_current == 1) {
      $("select.[id*=afrimesh|settings|radius|radtype]").html
        ("<option value = '1' selected>mysql</option> <option value = '2'>memcachedb</option>");
    } else {
      $("select.[id*=afrimesh|settings|radius|radtype]").html
      ("<option value = '1'>mysql</option> <option value = '2' selected>memcachedb</option>");
    }
    
    $("input.[id*=afrimesh|settings|radius|server]").css("background", "#FFAAAA");
    try {
      var status = afrimesh.customers.status();
      var select = afrimesh.customers.select();
      
      if (status[0].error) {
        console.debug("RADIUS server is unreachable. " + status[0].error);
        $("p.[id*=radius|server|error]").html("RADIUS server unreachable. " + status[0].error + ".");
        return;
      } else if (select[0].error) {
        console.debug("mysql database is inaccessible. " + select[0].error);
        $("p.[id*=radius|server|error]").html("mysql database inaccessible. " + select[0].error + ".");
        return;
      } else {
        $("p.[id*=radius|server|error]").html("");
        $("input.[id*=afrimesh|settings|radius|server]").css("background", "#AAFFAA");
      }
    } catch (error) {
      console.debug("Unexpected error while contacting RADIUS server: " + error);
      $("p.[id*=radius|server|error]").html("RADIUS server unreachable. Unknown reason.");
    }
  };
  
  
  /** create a map which can be used to set the router location --------- */
  function _LocationMap(id, longitude, latitude, extent, zoom, on_position) {
    
    var the_map = (function() { 
        var options = {
          projection        : epsg_900913,
          displayProjection : epsg_4326,
          units             : "m",
          numZoomLevels     : 20,
          restrictedExtent  : new OpenLayers.Bounds(),
          theme             : "style/map.default.css"
        };
        options.restrictedExtent.extend(LonLat(longitude - extent, latitude - extent));
        options.restrictedExtent.extend(LonLat(longitude + extent, latitude + extent));
        var map = new OpenLayers.Map(id, options);
        if (afrimesh.settings.map.server == "openstreetmap.org") { // TODO afrimesh.settings.map.server == afrimesh.settings.map.server.default
          map.addLayers([ new OpenLayers.Layer.OSM.CycleMap("Relief Map") ]);
        } else {
          map.addLayers([ new OpenLayers.Layer.OSM.LocalMap("Relief Map", "http://" + afrimesh.settings.map.server + "/tiles/") ]);
        }
        map.addControl(new OpenLayers.Control.Attribution());
        map.addControl(new OpenLayers.Control.MousePosition());
        map.addControl(new OpenLayers.Control.ScaleLine());
        map.setCenter(LonLat(longitude, latitude), zoom);
        if (!map.getCenter()) { map.zoomToMaxExtent(); }
        map.addLayer(new OpenLayers.Layer.Vector("Mesh Routers"));
        map.routers  = map.getLayersByName("Mesh Routers")[0];
        
        var dragger = new OpenLayers.Control.DragFeature(map.routers);
        if (on_position) {
          dragger.onComplete = on_position;
        } else { 
          dragger.onComplete = function(feature) {     // TODO - ultimately we want this to be just "feature.router.settings.location=" 
            var location = new OpenLayers.LonLat(feature.geometry.x, feature.geometry.y).transform(epsg_900913, epsg_4326);
            afrimesh.villagebus.uci.set.async(function (response) {
                console.debug("Updated router location for:" + feature.router.address);
              }, feature.router.address,     
              [ { config: "afrimesh", section: "location", option: "longitude", value: location.lon.toString() },
                { config: "afrimesh", section: "location", option: "latitude",  value: location.lat.toString() } ]);
          };
        }
        map.addControl(dragger);
        dragger.activate();
        return map;
      })(); /* create_map */
    
    
    /** 
     * Return the map feature representing the given router, creating it and 
     * adding it to the map if it doesn't exist yet.
     */
    this.routers = the_map.routers;
    this.router = function(router) {
      var feature = the_map.routers.getFeatureById(router.address);
      if (feature) {
        return feature;
      }
      return add_router(router);
    };
    function add_router(router) {
      var feature = new OpenLayers.Feature.Vector();
      feature.style = { fillColor: "black", 
                        fillOpacity: 0.5, 
                        strokeOpacity: 1.0,
                        strokeColor: "black",
                        strokeWidth: 1.0,
                        pointRadius: 10.0 };
      feature.id = router.address;
      feature.router = router;
      the_map.routers.addFeatures([feature]);
      afrimesh.villagebus.uci.get.async(function (config) {
          feature.geometry = new Point(config.afrimesh.location.longitude, config.afrimesh.location.latitude);
          the_map.routers.redraw();
        }, router.address, "afrimesh.location");
      return feature;
    };
    
    
  };
  LocationMap = _LocationMap;


  /** helpers ------------------------------------------------------------- */
  disable_key = function(key, event) {
    var pressed;      
    if (window.event) pressed = window.event.keyCode; // IE
    else              pressed = e.which;              // firefox      
    return (pressed != key);
  }

  
})();
